/*
 * This program is free software; you can redistribute it and/or modify it under the
 * terms of the GNU Lesser General Public License, version 2.1 as published by the Free Software
 * Foundation.
 *
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, you can obtain a copy at http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html
 * or from the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 *
 * Copyright (c) 2012 Pentaho Corporation..  All rights reserved.
 */

pentaho = pentaho || {};

pentaho.viz = typeof pentaho.viz == "undefined" ? {} : pentaho.viz;

pentaho.viz.util = typeof pentaho.viz.util == "undefined" ? {} : pentaho.viz.util;

pentaho.visualizations = typeof pentaho.visualizations == "undefined" ? [] : pentaho.visualizations;
 
pentaho.viz.util.HandleCommonOptions = pentaho.viz.util.HandleCommonOptions || function( viz, options ) {

    // check the options for settings (set by the caller)
    if( options ) {
        if( typeof options.labelColor != "undefined" ) {
            viz.labelColor = options.labelColor;
        }
        if( typeof options.labelStyle != "undefined" ) {
            viz.labelStyle = options.labelStyle;
        }
        if( typeof options.labelSize != "undefined" ) {
            viz.labelSize = options.labelSize;
        }
        if( typeof options.labelFontFamily != "undefined" ) {
            viz.labelFontFamily = options.labelFontFamily;
        }
        if( typeof options.backgroundType != "undefined" ) {
            viz.backgroundType = options.backgroundType;
        }
        if( typeof options.backgroundColor != "undefined" ) {
            viz.backgroundColor = options.backgroundColor;
        }
        if( typeof options.backgroundColorEnd != "undefined" ) {
            viz.backgroundColorEnd = options.backgroundColorEnd;
        }
        if( typeof options.reverseColors != 'undefined') {
            viz.reverseColors = options.reverseColors;
        }
        if( typeof options.scalingType != 'undefined') {
            this.scalingType = options.scalingType;
        }
        if( typeof options.pattern != 'undefined') {
            this.pattern = options.pattern;
        }
        if( typeof options.colorSet != 'undefined') {
            this.colorSet = options.colorSet;
        }
        if( typeof options.reverseColors != 'undefined') {
            this.reverseColors = options.reverseColors;
        }
    }

    // check the controller current visualizations arguments (set by the properties panel)
    if( viz.controller && viz.controller.currentViz && viz.controller.currentViz.args ) {
        var labelColor = viz.controller.currentViz.args.labelColor;
        if( labelColor ) { // TODO localize this?
            viz.labelColor = labelColor;
        }
        var labelFontFamily = viz.controller.currentViz.args.labelFontFamily;
        if( labelFontFamily && labelFontFamily != 'Default') { // TODO localize this?
            viz.labelFontFamily = labelFontFamily;
        }
        var labelSize = viz.controller.currentViz.args.labelSize;
        if( labelSize ) {
            viz.labelSize = labelSize;
        }
        var labelStyle = viz.controller.currentViz.args.labelStyle;
        if( labelStyle && labelStyle != 'PLAIN') {
            viz.labelStyle = labelStyle;
        }
        var backgroundType = viz.controller.currentViz.args.backgroundType;
        if( backgroundType ) {
            viz.backgroundType = backgroundType;
        }
        var backgroundColor = viz.controller.currentViz.args.backgroundColor;
        if( backgroundColor ) {
            viz.backgroundColor = backgroundColor;
        }
        var backgroundColorEnd = viz.controller.currentViz.args.backgroundColorEnd;
        if( backgroundColorEnd ) {
            viz.backgroundColorEnd = backgroundColorEnd;
        }
        if( typeof viz.controller.currentViz.args.reverseColors != 'undefined' ) {
            viz.reverseColors = viz.controller.currentViz.args.reverseColors;
            options.reverseColors = viz.reverseColors;
        }
        if( typeof viz.controller.currentViz.args.pattern != 'undefined' || typeof viz.controller.currentViz.args.colorSet != 'undefined' ) {
            
            if( typeof viz.controller.currentViz.args.pattern != 'undefined' ) {
                viz.pattern = viz.controller.currentViz.args.pattern;
            } else {
                viz.pattern = 'GRADIENT';
            }
            viz.colorMode = viz.pattern == 'PIE' ? 'top' : 'gradient';
            viz.scalingType = viz.pattern == 'GRADIENT' ? 'linear' : 'steps'
            if( typeof viz.controller.currentViz.args.colorSet != 'undefined' ) {
                viz.colorSet = viz.controller.currentViz.args.colorSet;
            } else {
                viz.colorSet = 'ryg';
            }
            viz.options.colorMode = viz.colorMode;
            viz.options.scalingType = viz.scalingType;
            viz.options.pattern = viz.pattern;
            viz.options.colorSet = viz.colorSet;
        }
    }
    
    // check Analyzer's report document (yuck)
    if( cv && cv.rptEditor && cv.rptEditor.report && cv.rptEditor.report.reportDoc ) {
        var labelColor = cv.rptEditor.report.reportDoc.getChartOption('labelColor');
        if( labelColor ) {
            viz.labelColor = labelColor;
            viz.controller.currentViz.args.labelColor = labelColor;
            options.labelColor = labelColor;
        }
        var labelFontFamily = cv.rptEditor.report.reportDoc.getChartOption('labelFontFamily');
        if( labelFontFamily && labelFontFamily != 'Default') { // TODO localize this?
            viz.labelFontFamily = labelFontFamily;
            viz.controller.currentViz.args.labelFontFamily = labelFontFamily;
            options.labelFontFamily = labelFontFamily;
        }
        var labelSize = cv.rptEditor.report.reportDoc.getChartOption('labelSize');
        if( labelSize ) {
            viz.labelSize = labelSize;
            options.labelSize = labelSize;
            viz.controller.currentViz.args.labelSize = labelSize;
        }
        var labelStyle = cv.rptEditor.report.reportDoc.getChartOption('labelStyle');
        if( labelStyle && labelStyle != 'PLAIN') {
            viz.labelStyle = labelStyle;
            viz.controller.currentViz.args.labelStyle = labelStyle;
            options.labelStyle = labelStyle;
        }
        var backgroundType = cv.rptEditor.report.reportDoc.getChartOption('backgroundFill');
        if( backgroundType ) {
            viz.backgroundType = backgroundType;
            viz.controller.currentViz.args.backgroundType = backgroundType;
            options.backgroundType = backgroundType;
        }
        var backgroundColor = cv.rptEditor.report.reportDoc.getChartOption('backgroundColor');
        if( backgroundColor ) {
            viz.backgroundColor = backgroundColor;
            viz.controller.currentViz.args.backgroundColor = backgroundColor;
            options.backgroundColor = backgroundColor;
        }
        var backgroundColorEnd = cv.rptEditor.report.reportDoc.getChartOption('backgroundColorEnd');
        if( backgroundColorEnd ) {
            viz.backgroundColorEnd = backgroundColorEnd;
            viz.controller.currentViz.args.backgroundColorEnd = backgroundColorEnd;
            options.backgroundColorEnd = backgroundColorEnd;
        }
    }
    viz.labelFontStr = '';
    
    if( viz.labelStyle && viz.labelStyle != 'PLAIN') {
        viz.labelFontStr += viz.labelStyle + " ";
    }
    viz.labelFontStr += viz.labelSize+"pt ";
    if( viz.labelFontFamily && viz.labelFontFamily != 'Default') { // TODO localize this?
        viz.labelFontStr += viz.labelFontFamily;
    } else {
        viz.labelFontStr += "Arial";
    }
}        

pentaho.viz.util.ApplyBackground = pentaho.viz.util.ApplyBackground || function( element, options ) {
        
    if( options.backgroundType == 'NONE' ) {
        element.style.background = 'none';
        element.style.backgroundColor = '';
    }
    else if( options.backgroundType == 'SOLID' ) {
        element.style.backgroundColor = options.backgroundColor;
    }
    else if( options.backgroundType == 'GRADIENT' ) {
        delete element.style.backgroundColor;            
        var origBack = element.style.background;
        if( element.style.background ) {
            var back = "-moz-linear-gradient(top, "+options.backgroundColor+", "+options.backgroundColorEnd+")";
            element.style.background = back;
            if( !element.style.background || element.style.background == origBack ) {
                back = "-webkit-gradient(linear, left top, left bottom, from("+options.backgroundColor+"), to("+options.backgroundColorEnd+"))";
                element.style.background = back;
            }
        }
        // detect IE
        var browserName=navigator.appName;
        if (browserName=="Microsoft Internet Explorer") {
            back = "progid:DXImageTransform.Microsoft.Gradient(startColorstr="+options.backgroundColor+", endColorstr="+options.backgroundColorEnd+")";
            element.style.filter = back;
        }
    }

}

pentaho.viz.util.setupColorRanges = pentaho.viz.util.setupColorRanges || function( viz ) {


    if( viz.colorSet == 'ryg' ) {
        if( viz.pattern == '5-COLOR' ) {
            viz.colorRange = ["#ff0000","#ffbf3f","#ffff00","#bfdf3f","#008000"];
        } else {
            viz.colorRange = ["#ff0000","#ffff00","#008000"];
        }
    }
    else if( viz.colorSet == 'ryb' ) {
        if( viz.pattern == '5-COLOR' ) {
            viz.colorRange = ["#ff0000","#ffbf3f","#ffff00","#dcddde","#4bb6e4"];
        } else {
            viz.colorRange = ["#ff0000","#ffff00","#4bb6e4"];
        }
    }
    else if( viz.colorSet == 'blue' ) {
        if( viz.pattern == '5-COLOR' ) {
            viz.colorRange = ["#cbe7ff","#99d0ff","#33a1ff","#006ecc","#0345a9"];
        } else {
            viz.colorRange = ["#cbe7ff","#33a1ff","#0345a9"];
        }
    }
    else if( viz.colorSet == 'gray' ) {
        if( viz.pattern == '5-COLOR' ) {
            viz.colorRange = ["#e6e6e6","#cccccc","#999999","#666666","#333333"];
        } else {
            viz.colorRange = ["#e6e6e6","#999999","#333333"];
        }
    }
    if( viz.reverseColors ) {
        var range = viz.colorRange;
        viz.colorRange = [];
        for( var idx=range.length-1; idx>=0; idx-- ) {
            viz.colorRange.push(range[idx]);
        }
    }
};
    
pentaho.viz.util.getClientWidth = pentaho.viz.util.getClientWidth || function(viz) {
      return viz.element.offsetWidth;
};

pentaho.viz.util.getClientHeight = pentaho.viz.util.getClientHeight || function(viz) {
      return viz.element.offsetHeight;
};

pentaho.viz.util.getRgbGradientFromMultiColorHex = pentaho.viz.util.getRgbGradientFromMultiColorHex || function(value, min, max, colors) {

    var steps = colors.length-1;
    var range = max-min;

    if(range <= 0) {
        var start = colors.length-1;
        var end = start;
    } else {
        var start = Math.floor(((value-min)/range) * steps);
        var end = Math.ceil(((value-min)/range) * steps);
    }
    var color1 = pentaho.VizController.convertToRGB(colors[start]);
    var color2 = pentaho.VizController.convertToRGB(colors[end]);

    var rangeMin = (start / steps) * range + min;
    var rangeMax = (end / steps) * range + min;

    var inRange
    if(rangeMin == rangeMax){
        inRange = 1;
    } else {
        inRange = (value-rangeMin)/(rangeMax-rangeMin);
    }
    var cols = new Array(3);
    cols[0] = Math.floor( inRange * (color2[0] - color1[0]) + color1[0] );
    cols[1] = Math.floor( inRange * (color2[1] - color1[1]) + color1[1] );
    cols[2] = Math.floor( inRange * (color2[2] - color1[2]) + color1[2] );
    return pentaho.VizController.getRrbColor(cols[0], cols[1], cols[2]);
}    

pentaho.viz.util.ClearSelections = pentaho.viz.util.ClearSelections || function( ) {
    // TODO convert this call when Analyzer has a generic way to do this
    if( cv && cv.getActiveReport() ) {
        cv.getActiveReport().hideSelectionFilterButtons();
    }
}

pentaho.viz.util.CreateSelectionTable = pentaho.viz.util.CreateSelectionTable || function( table, dataTable, labelFontStr, labelColor ) {
    console.log('pentaho.viz.util.CreateSelectionTable()');

    table.id = 'datalist';
    table.className = 'datatable';
    table.cellPadding = "0";
    table.cellSpacing = "0";
    
    var row = table.insertRow(-1);
    var cell = row.insertCell(0);
    cell.innerHTML = "#";  
    cell.className = 'tableheader alignRight';
    cell.style.font = labelFontStr;
    cell.style.color = labelColor;
    cell.style.borderBottom = "2px solid "+labelColor;
    var measureNo = 0;
    for( var colNo=0; colNo<dataTable.getNumberOfColumns(); colNo++) {
        var cell = row.insertCell(colNo+1);
        var sortable = '';
        if( dataTable.getColumnType(colNo).toUpperCase() == 'NUMBER' ) {
            cell.className = 'tableheader alignRight';
            sortable = ' <span onClick="crossFilterWrapper.setSort('+measureNo+')"><></span>';
            measureNo++;
        } else {
            cell.className = 'tableheader';
        }
        cell.style.font = labelFontStr;
        cell.style.color = labelColor;
        cell.style.borderBottom = "2px solid "+labelColor;
        cell.innerHTML = dataTable.getColumnLabel(colNo) + sortable;  
    }
}

pentaho.viz.util.UpdateSelectionTable = pentaho.viz.util.UpdateSelectionTable || function( table, dataTable, rows, labelFontStr, labelColor ) {
    console.log('pentaho.viz.util.UpdateSelectionTable()');

    while( table.rows.length>1) {
        table.deleteRow(1);
    }

    var colCount = dataTable.getNumberOfColumns();
    var rowCount = rows.length;
    for( var idx=0; idx<rowCount; idx++ ) {
        var rowIdx = rows[idx].rowIdx;
        var row = table.insertRow(idx+1);
        var cell = row.insertCell(0);
        cell.innerHTML = ''+(idx+1);
        var rowClass = idx % 2 ? ' oddRow' : '';
        var className = 'tableCell' + rowClass;
        if( idx == rowCount-1 ) {
            className += ' lastRow';
        }
        cell.className = className+' alignRight';
        cell.style.font = labelFontStr;
        cell.style.color = labelColor;
        for( var colNo=0; colNo<colCount; colNo++) {
            var cell = row.insertCell(colNo+1);
            var value = dataTable.getFormattedValue(rowIdx, colNo);
            if( typeof value == 'undefined') {
                value = dataTable.getValue(rowIdx, colNo);
            }
            cell.innerHTML = value;  
            if( dataTable.getColumnType(colNo).toUpperCase() == 'NUMBER' ) {
                className += ' alignRight';
            }
            if( colNo == colCount-1 ) {
                className += ' lastColumn';
            }
            cell.className = className;
            cell.style.font = labelFontStr;
            cell.style.color = labelColor;
        }
    }
  }
    
pentaho.viz.util.RegisterVisualization = pentaho.viz.util.RegisterVisualization || function( identity ) {

  analyzerPlugins = typeof analyzerPlugins == "undefined" ? [] : analyzerPlugins;

  analyzerPlugins.push(
      {
        pluginId: identity.pluginId,
        vizId: identity.vizId,
                
        init:function () {
          dojo.require("pentaho.common.Messages");
          pentaho.common.Messages.addUrlBundle(this.pluginId,CONTEXT_PATH+'i18n?plugin='+this.pluginId+'&name=resources/messages/messages');

            if( pentaho.palettes && pentaho.palettes[0].name == 'palette 1' ) {
                pentaho.palettes.splice( 0, 0, {
                    name: 'default',
                    colors: [
                    "#0d8ecf",
                    "#b0de09",
                    "#fcd202",
                    "#ff6600",
                    "#cd0d74",
                    "#cc0000",
                    "#00cc00",
                    "#650cd0",
                    "#3a3a3a",
                    "#7272e3",
                    "#79c1e4",
                    "#d3ed77",
                    "#fde673",
                    "#ffaa72",
                    "#e379b2",
                    "#e37272",
                    "#72e372",
                    "#aa79e5",
                    "#929292",
                    "#00007a",
                    "#07547b",
                    "#698405",
                    "#967c01",
                    "#983b00",
                    "#7a0745",
                    "#7a0000",
                    "#007a00",
                    "#3c077b",
                    "#000000"]
                } );
            }

          // Register types to display in Analyzer
          if(!cv.pentahoVisualizations){
            cv.pentahoVisualizations = [];
          }
          var vis = pentaho.visualizations.getById(this.vizId);
          if( vis ) {
            cv.pentahoVisualizations.push(vis);
          } else {
            console.log('ERROR: could not find visualization - '+this.vizId );
          }
          vis.name = pentaho.common.Messages.getString(this.vizId);

          cv.pentahoVisualizationHelpers[this.vizId] = {
            vizId: this.vizId,
            previousAction: null,
            canRefreshReport:function(report) {

              var vis = pentaho.visualizations.getById(this.vizId);
              if(vis.helper && vis.helper.canRefreshReport) {
                return vis.helper.canRefreshReport(report);
              }
            },
            generateOptionsFromAnalyzerState:function (report) {
              var userDefinedOpts = {};

              // set the last action taken
              if(report.visualization.args.configAction !== 'undefined' && report.visualization.args.configAction != null) {
                report.visualization.args.configAction = null;
              } else if(report.history.current() != null) {
                var action = report.history.current().action;
                this.previousAction = action;
              }
              return userDefinedOpts;
            }
          };

          dojo.declare("analyzer."+this.vizId+"Config", [analyzer.ColorConfiguration], {
            onModelEvent:function (config, item, eventName, args) {
              if (eventName == "value") {
                // works by convention where the ids of the data req items match the property names
                this.report.visualization.args[item.id] = args.newVal;
                if(typeof(args.prevVal) != "undefined" && args.prevVal != args.newVal){
                  this.report.visualization.args.configAction = item.id + "Changed";
                }
              }
              this.inherited(arguments); // Let super class handle the insertAt and removedGem events
            },
            _setScalingType:function (scalingType) {
              this.report.visualization.args.scalingType = scalingType;
            },
            _setColorRange:function (range) {
              this.report.visualization.args.colorRange = range;
            },
            getConfiguration:function () {
              var config = this.inherited(arguments);
              return config;
            },
            updateConfiguration: function(config){
            }

          });
          analyzer.LayoutPanel.configurationManagers["JSON_"+this.vizId] = analyzer[this.vizId+'Config'];

        }
      }
  );

}

